@@ -47,14 +47,16 @@ import random
 import warnings
 from multiprocessing import cpu_count
 import os
+import sys
 from re import search
 from datetime import datetime
 from time import sleep
 from tempfile import mkdtemp
 from shutil import rmtree
+import platform
 
-from sklearn.datasets import load_digits, load_boston, make_classification
-from sklearn.model_selection import train_test_split, cross_val_score, GroupKFold
+from sklearn.datasets import load_digits, load_boston, make_classification, make_regression
+from sklearn import model_selection
 from joblib import Memory
 from sklearn.metrics import make_scorer, roc_auc_score
 from sklearn.base import BaseEstimator, ClassifierMixin, RegressorMixin, TransformerMixin
@@ -67,7 +69,7 @@ from deap.tools import ParetoFront
 from nose.tools import nottest, assert_raises, assert_not_equal, assert_greater_equal, assert_equal, assert_in
 from driver_tests import captured_output
 
-train_test_split = nottest(train_test_split)
+train_test_split = nottest(model_selection.train_test_split)
 
 with warnings.catch_warnings():
     warnings.simplefilter('ignore')
@@ -102,9 +104,31 @@ training_features_r, testing_features_r, training_target_r, testing_target_r = \
 
 # Set up a small test dataset
 
-pretest_X, pretest_y = make_classification(n_samples=100, n_features=10, random_state=42)
-# Set up pandas DataFrame for testing
+pretest_X, pretest_y = make_classification(
+                                            n_samples=100,
+                                            n_features=10,
+                                            random_state=42)
 
+pretest_X_reg, pretest_y_reg = make_regression(
+                                            n_samples=100,
+                                            n_features=10,
+                                            random_state=42
+                                            )
+
+# Set up artificial groups
+n_datapoints = pretest_y.shape[0]
+groups = np.random.randint(low=0, high=n_datapoints//2, size=n_datapoints)
+
+# Set up custom cv split generators
+custom_cvs = [
+    model_selection.LeaveOneGroupOut().split(X=pretest_X, y=pretest_y, groups=groups),
+    model_selection.LeavePGroupsOut(2).split(X=pretest_X, y=pretest_y, groups=groups),
+    model_selection.RepeatedStratifiedKFold(),
+    5,
+    10,
+    ]
+
+# Set up pandas DataFrame for testing
 input_data = pd.read_csv(
     'tests/tests.csv',
     sep=',',
@@ -142,7 +166,8 @@ def test_init_custom_parameters():
         verbosity=1,
         random_state=42,
         disable_update_check=True,
-        warm_start=True
+        warm_start=True,
+        log_file=None
     )
 
     assert tpot_obj.population_size == 500
@@ -155,6 +180,7 @@ def test_init_custom_parameters():
     assert tpot_obj.max_time_mins is None
     assert tpot_obj.warm_start is True
     assert tpot_obj.verbosity == 1
+    assert tpot_obj.log_file == None
 
     tpot_obj._fit_init()
 
@@ -166,7 +192,14 @@ def test_init_custom_parameters():
     assert tpot_obj._optimized_pipeline_score == None
     assert tpot_obj.fitted_pipeline_ == None
     assert tpot_obj._exported_pipeline_text == []
+    assert tpot_obj.log_file == sys.stdout
 
+def test_init_custom_progress_file():
+    """ Assert that TPOT has right file handler to save progress. """
+    file_name = "progress.txt"
+    file_handle = open(file_name, "w")
+    tpot_obj = TPOTClassifier(log_file=file_handle)
+    assert tpot_obj.log_file == file_handle
 
 def test_init_default_scoring():
     """Assert that TPOT intitializes with the correct default scoring function."""
@@ -335,16 +368,38 @@ def test_timeout():
     tpot_obj._optimized_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
     tpot_obj.fitted_pipeline_ = tpot_obj._toolbox.compile(expr=tpot_obj._optimized_pipeline)
     # test _wrapped_cross_val_score with cv=20 so that it is impossible to finish in 1 second
+    cv = model_selection.KFold(n_splits=20).split(
+        X=training_features_r,
+        y=training_target_r,
+    )
+    cv = model_selection._split.check_cv(cv, training_target_r, classifier=False)
     return_value = _wrapped_cross_val_score(tpot_obj.fitted_pipeline_,
                                             training_features_r,
                                             training_target_r,
-                                            cv=20,
+                                            cv=cv,
                                             scoring_function='neg_mean_squared_error',
                                             sample_weight=None,
                                             groups=None,
                                             timeout=1)
     assert return_value == "Timeout"
 
+def test_custom_cv_test_generator():
+    """Check that custom cv generators processed correctly.
+    """
+    def check_custom_cv(_cv):
+        tpot_obj = TPOTClassifier(
+            random_state=42,
+            population_size=1,
+            offspring_size=2,
+            generations=1,
+            verbosity=0,
+            cv=_cv,
+            n_jobs=-1  # ensure pickling / parallelization
+        )
+        tpot_obj.fit(pretest_X, pretest_y)
+
+    for cv in custom_cvs:
+        yield check_custom_cv, cv
 
 def test_invalid_pipeline():
     """Assert that _wrapped_cross_val_score return -float(\'inf\') with a invalid_pipeline"""
@@ -357,11 +412,20 @@ def test_invalid_pipeline():
     )
     tpot_obj._optimized_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
     tpot_obj.fitted_pipeline_ = tpot_obj._toolbox.compile(expr=tpot_obj._optimized_pipeline)
-    # test _wrapped_cross_val_score with cv=20 so that it is impossible to finish in 1 second
+
+    cv = model_selection.KFold().split(
+        X=training_features,
+        y=training_target,
+    )
+    cv = model_selection._split.check_cv(
+        cv,
+        training_target,
+        classifier=False  # choice of classifier vs. regressor here is arbitrary
+    )
     return_value = _wrapped_cross_val_score(tpot_obj.fitted_pipeline_,
                                             training_features,
                                             training_target,
-                                            cv=5,
+                                            cv=cv,
                                             scoring_function='accuracy',
                                             sample_weight=None,
                                             groups=None,
@@ -573,7 +637,12 @@ def test_score_3():
 
     # Get score from TPOT
     score = tpot_obj.score(testing_features_r, testing_target_r)
-    assert np.allclose(known_score, score)
+    # On some non-amd64 systems such as arm64, a resulting score of
+    # 0.8207525232725118 was observed, so we need to add a tolerance there
+    if platform.machine() != 'amd64':
+        assert np.allclose(known_score, score, rtol=0.03)
+    else:
+        assert np.allclose(known_score, score)
 
 
 def test_sample_weight_func():
@@ -604,13 +673,13 @@ def test_sample_weight_func():
     training_target_r_weight_dict = set_sample_weight(tpot_obj.fitted_pipeline_.steps, training_target_r_weight)
 
     np.random.seed(42)
-    cv_score1 = cross_val_score(tpot_obj.fitted_pipeline_, training_features_r, training_target_r, cv=3, scoring='neg_mean_squared_error')
+    cv_score1 = model_selection.cross_val_score(tpot_obj.fitted_pipeline_, training_features_r, training_target_r, cv=3, scoring='neg_mean_squared_error')
 
     np.random.seed(42)
-    cv_score2 = cross_val_score(tpot_obj.fitted_pipeline_, training_features_r, training_target_r, cv=3, scoring='neg_mean_squared_error')
+    cv_score2 = model_selection.cross_val_score(tpot_obj.fitted_pipeline_, training_features_r, training_target_r, cv=3, scoring='neg_mean_squared_error')
 
     np.random.seed(42)
-    cv_score_weight = cross_val_score(tpot_obj.fitted_pipeline_, training_features_r, training_target_r, cv=3, scoring='neg_mean_squared_error', fit_params=training_target_r_weight_dict)
+    cv_score_weight = model_selection.cross_val_score(tpot_obj.fitted_pipeline_, training_features_r, training_target_r, cv=3, scoring='neg_mean_squared_error', fit_params=training_target_r_weight_dict)
 
     np.random.seed(42)
     tpot_obj.fitted_pipeline_.fit(training_features_r, training_target_r, **training_target_r_weight_dict)
@@ -621,7 +690,12 @@ def test_sample_weight_func():
 
     assert np.allclose(cv_score1, cv_score2)
     assert not np.allclose(cv_score1, cv_score_weight)
-    assert np.allclose(known_score, score)
+    # On some non-amd64 systems such as arm64, a resulting score of
+    # 0.8207525232725118 was observed, so we need to add a tolerance there
+    if platform.machine() != 'amd64':
+        assert np.allclose(known_score, score, rtol=0.01)
+    else:
+        assert np.allclose(known_score, score)
 
 
 def test_template_1():
@@ -732,7 +806,7 @@ def test_fit_GroupKFold():
         generations=1,
         verbosity=0,
         config_dict='TPOT light',
-        cv=GroupKFold(n_splits=2),
+        cv=model_selection.GroupKFold(n_splits=2),
     )
     tpot_obj.fit(training_features, training_target, groups=groups)
 
@@ -1020,6 +1094,21 @@ def test_fit_6():
     assert not (tpot_obj._start_datetime is None)
 
 
+def test_fit_7():
+    """Assert that the TPOT fit function provides an optimized pipeline."""
+    tpot_obj = TPOTRegressor(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0
+    )
+    tpot_obj.fit(pretest_X_reg, pretest_y_reg)
+
+    assert isinstance(tpot_obj._optimized_pipeline, creator.Individual)
+    assert not (tpot_obj._start_datetime is None)
+
+
 def test_memory():
     """Assert that the TPOT fit function runs normally with memory=\'auto\'."""
     tpot_obj = TPOTClassifier(
@@ -1152,7 +1241,7 @@ def test_check_periodic_pipeline():
     )
     tpot_obj.fit(training_features, training_target)
     with closing(StringIO()) as our_file:
-        tpot_obj._file = our_file
+        tpot_obj.log_file = our_file
         tpot_obj.verbosity = 3
         tpot_obj._last_pipeline_write = datetime.now()
         sleep(0.11)
@@ -1196,7 +1285,7 @@ def test_save_periodic_pipeline():
     )
     tpot_obj.fit(training_features, training_target)
     with closing(StringIO()) as our_file:
-        tpot_obj._file = our_file
+        tpot_obj.log_file = our_file
         tpot_obj.verbosity = 3
         tpot_obj._last_pipeline_write = datetime.now()
         sleep(0.11)
@@ -1226,7 +1315,7 @@ def test_save_periodic_pipeline_2():
     )
     tpot_obj.fit(training_features, training_target)
     with closing(StringIO()) as our_file:
-        tpot_obj._file = our_file
+        tpot_obj.log_file = our_file
         tpot_obj.verbosity = 3
         tpot_obj._last_pipeline_write = datetime.now()
         sleep(0.11)
@@ -1257,7 +1346,7 @@ def test_check_periodic_pipeline_3():
     )
     tpot_obj.fit(training_features, training_target)
     with closing(StringIO()) as our_file:
-        tpot_obj._file = our_file
+        tpot_obj.log_file = our_file
         tpot_obj.verbosity = 3
         tpot_obj._exported_pipeline_text = []
         tpot_obj._last_pipeline_write = datetime.now()
@@ -1383,9 +1472,9 @@ def test_evaluated_individuals_():
         operator_count = tpot_obj._operator_count(deap_pipeline)
 
         try:
-            cv_scores = cross_val_score(sklearn_pipeline, training_features, training_target, cv=5, scoring='accuracy', verbose=0)
+            cv_scores = model_selection.cross_val_score(sklearn_pipeline, training_features, training_target, cv=5, scoring='accuracy', verbose=0)
             mean_cv_scores = np.mean(cv_scores)
-        except Exception as e:
+        except Exception:
             mean_cv_scores = -float('inf')
         assert np.allclose(tpot_obj.evaluated_individuals_[pipeline_string]['internal_cv_score'], mean_cv_scores)
         assert np.allclose(tpot_obj.evaluated_individuals_[pipeline_string]['operator_count'], operator_count)
@@ -1431,15 +1520,15 @@ def test_evaluate_individuals():
         try:
             with warnings.catch_warnings():
                 warnings.simplefilter('ignore')
-                cv_scores = cross_val_score(sklearn_pipeline,
-                                            training_features,
-                                            training_target,
-                                            cv=5,
-                                            scoring='accuracy',
-                                            verbose=0,
-                                            error_score='raise')
+                cv_scores = model_selection.cross_val_score(sklearn_pipeline,
+                                                            training_features,
+                                                            training_target,
+                                                            cv=5,
+                                                            scoring='accuracy',
+                                                            verbose=0,
+                                                            error_score='raise')
             mean_cv_scores = np.mean(cv_scores)
-        except Exception as e:
+        except Exception:
             mean_cv_scores = -float('inf')
 
         assert isinstance(deap_pipeline, creator.Individual)
@@ -1473,15 +1562,15 @@ def test_evaluate_individuals_2():
         try:
             with warnings.catch_warnings():
                 warnings.simplefilter('ignore')
-                cv_scores = cross_val_score(sklearn_pipeline,
-                                            training_features,
-                                            training_target,
-                                            cv=5,
-                                            scoring='accuracy',
-                                            verbose=0,
-                                            error_score='raise')
+                cv_scores = model_selection.cross_val_score(sklearn_pipeline,
+                                                            training_features,
+                                                            training_target,
+                                                            cv=5,
+                                                            scoring='accuracy',
+                                                            verbose=0,
+                                                            error_score='raise')
             mean_cv_scores = np.mean(cv_scores)
-        except Exception as e:
+        except Exception:
             mean_cv_scores = -float('inf')
 
         assert isinstance(deap_pipeline, creator.Individual)
@@ -1500,7 +1589,7 @@ def test_update_pbar():
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
-        tpot_obj._file=our_file
+        tpot_obj.log_file=our_file
         tpot_obj._pbar = tqdm(total=10, disable=False, file=our_file)
         tpot_obj._update_pbar(pbar_num=2, pbar_msg="Test Warning Message")
         our_file.seek(0)
@@ -1519,7 +1608,7 @@ def test_update_val():
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
-        tpot_obj._file=our_file
+        tpot_obj.log_file=our_file
         tpot_obj._pbar = tqdm(total=10, disable=False, file=our_file)
         result_score_list = []
         result_score_list = tpot_obj._update_val(0.9999, result_score_list)
@@ -1566,9 +1655,9 @@ def test_preprocess_individuals():
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
-        tpot_obj._file=our_file
+        tpot_obj.log_file=our_file
         tpot_obj._pbar = tqdm(total=2, disable=False, file=our_file)
-        operator_counts, eval_individuals_str, sklearn_pipeline_list, stats_dicts = \
+        operator_counts, eval_individuals_str, sklearn_pipeline_list, _ = \
                                 tpot_obj._preprocess_individuals(individuals)
         our_file.seek(0)
         assert_in("Pipeline encountered that has previously been evaluated", our_file.read())
@@ -1612,9 +1701,9 @@ def test_preprocess_individuals_2():
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
-        tpot_obj._file=our_file
+        tpot_obj.log_file=our_file
         tpot_obj._pbar = tqdm(total=3, disable=False, file=our_file)
-        operator_counts, eval_individuals_str, sklearn_pipeline_list, stats_dicts = \
+        operator_counts, eval_individuals_str, sklearn_pipeline_list, _ = \
                                 tpot_obj._preprocess_individuals(individuals)
         our_file.seek(0)
 
@@ -1659,14 +1748,32 @@ def test_preprocess_individuals_3():
     # reset verbosity = 3 for checking pbar message
 
     with closing(StringIO()) as our_file:
-        tpot_obj._file=our_file
+        tpot_obj.log_file=our_file
         tpot_obj._lambda=4
         tpot_obj._pbar = tqdm(total=2, disable=False, file=our_file)
         tpot_obj._pbar.n = 2
-        operator_counts, eval_individuals_str, sklearn_pipeline_list, stats_dicts = \
+        _, _, _, _ = \
                                 tpot_obj._preprocess_individuals(individuals)
         assert tpot_obj._pbar.total == 6
 
+def test__init_pretest():
+    """Assert that the init_pretest function produces a sample with all labels"""
+    tpot_obj = TPOTClassifier(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0,
+        config_dict='TPOT light'
+    )
+    tpot_obj._fit_init()
+
+    np.random.seed(seed=42)
+    features = np.random.rand(10000,2)
+    target = np.random.binomial(1,0.01,(10000,1))
+
+    tpot_obj._init_pretest(features, target)
+    assert(np.unique(tpot_obj.pretest_y).size == np.unique(target).size)
 
 def test_check_dataset():
     """Assert that the check_dataset function returns feature and target as expected."""
@@ -1697,7 +1804,7 @@ def test_check_dataset_2():
     )
     tpot_obj._fit_init()
     test_sample_weight = list(range(1, len(training_target)+1))
-    ret_features, ret_target = tpot_obj._check_dataset(training_features, training_target, test_sample_weight)
+    _, _ = tpot_obj._check_dataset(training_features, training_target, test_sample_weight)
     test_sample_weight[0] = 'opps'
 
     assert_raises(ValueError, tpot_obj._check_dataset, training_features, training_target, test_sample_weight)
@@ -1715,7 +1822,7 @@ def test_check_dataset_3():
     )
     tpot_obj._fit_init()
     test_sample_weight = list(range(1, len(training_target)+1))
-    ret_features, ret_target = tpot_obj._check_dataset(training_features, training_target, test_sample_weight)
+    _, _ = tpot_obj._check_dataset(training_features, training_target, test_sample_weight)
     test_sample_weight[0] = np.nan
 
     assert_raises(ValueError, tpot_obj._check_dataset, training_features, training_target, test_sample_weight)
@@ -2086,7 +2193,7 @@ def test_mate_operator():
     tpot_obj.evaluated_individuals_[str(ind1)] = (2, 0.99)
     tpot_obj.evaluated_individuals_[str(ind2)] = (2, 0.99)
 
-    offspring1, offspring2 = tpot_obj._mate_operator(ind1, ind2)
+    offspring1, _ = tpot_obj._mate_operator(ind1, ind2)
     expected_offspring1 = (
         'KNeighborsClassifier('
         'BernoulliNB(input_matrix, BernoulliNB__alpha=10.0, BernoulliNB__fit_prior=False), '
