@@ -105,6 +105,17 @@ regressor_config_dict = {
         'objective': ['reg:squarederror']
     },
 
+    'sklearn.linear_model.SGDRegressor': {
+        'loss': ['squared_loss', 'huber', 'epsilon_insensitive'],
+        'penalty': ['elasticnet'],
+        'alpha': [0.0, 0.01, 0.001] ,
+        'learning_rate': ['invscaling', 'constant'] ,
+        'fit_intercept': [True, False],
+        'l1_ratio': [0.25, 0.0, 1.0, 0.75, 0.5],
+        'eta0': [0.1, 1.0, 0.01],
+        'power_t': [0.5, 0.0, 1.0, 0.1, 100.0, 10.0, 50.0]
+    },
+
     # Preprocesssors
     'sklearn.preprocessing.Binarizer': {
         'threshold': np.arange(0.0, 1.01, 0.05)
