@@ -25,6 +25,11 @@ License along with TPOT. If not, see <http://www.gnu.org/licenses/>.
 
 import numpy as np
 from sklearn.base import BaseEstimator, ClassifierMixin, RegressorMixin, TransformerMixin
+from sklearn.gaussian_process.kernels import Kernel
+try:
+    from sklearn.feature_selection._base import SelectorMixin
+except ImportError:
+    from sklearn.feature_selection.base import SelectorMixin
 import inspect
 
 
@@ -173,18 +178,23 @@ def TPOTOperatorClassFactory(opsourse, opdict, BaseClass=Operator, ArgBaseClass=
         return None, None
     else:
         # define if the operator can be the root of a pipeline
-        if issubclass(op_obj, ClassifierMixin) or issubclass(op_obj, RegressorMixin):
+        if issubclass(op_obj, ClassifierMixin):
             class_profile['root'] = True
-            optype = "Classifier or Regressor"
-        else:
-            optype = "Preprocessor or Selector"
+            optype = "Classifier"
+        elif issubclass(op_obj, RegressorMixin):
+            class_profile['root'] = True
+            optype = "Regressor"
+        if issubclass(op_obj, TransformerMixin):
+            optype = "Transformer"
+        if issubclass(op_obj, SelectorMixin):
+            optype = "Selector"
 
         @classmethod
         def op_type(cls):
             """Return the operator type.
 
             Possible values:
-                "Classifier", "Regressor", "Selector", "Preprocessor"
+                "Classifier", "Regressor", "Selector", "Transformer"
             """
             return optype
 
@@ -203,7 +213,7 @@ def TPOTOperatorClassFactory(opsourse, opdict, BaseClass=Operator, ArgBaseClass=
                 for dkey, dval in prange.items():
                     dep_import_str, dep_op_str, dep_op_obj = source_decode(dkey, verbose=verbose)
                     if dep_import_str in import_hash:
-                        import_hash[import_str].append(dep_op_str)
+                        import_hash[dep_import_str].append(dep_op_str)
                     else:
                         import_hash[dep_import_str] = [dep_op_str]
                     dep_op_list[pname] = dep_op_str
@@ -233,7 +243,7 @@ def TPOTOperatorClassFactory(opsourse, opdict, BaseClass=Operator, ArgBaseClass=
                 operator
 
             """
-            return ([np.ndarray] + arg_types, np.ndarray)
+            return ([np.ndarray] + arg_types, np.ndarray) # (input types, return types)
 
         class_profile['parameter_types'] = parameter_types
 
@@ -283,7 +293,8 @@ def TPOTOperatorClassFactory(opsourse, opdict, BaseClass=Operator, ArgBaseClass=
                         if issubclass(doptype, BaseEstimator) or \
                             issubclass(doptype, ClassifierMixin) or \
                             issubclass(doptype, RegressorMixin) or \
-                            issubclass(doptype, TransformerMixin):
+                            issubclass(doptype, TransformerMixin) or \
+                            issubclass(doptype, Kernel):
                             arg_value = "{}({})".format(dep_op_str, ", ".join(dep_op_arguments[dep_op_str]))
                     tmp_op_args.append("{}={}".format(dep_op_pname, arg_value))
             op_arguments = tmp_op_args + op_arguments
